#!/usr/bin/env python3
'''Compare symbols within two object (or archive) files'''
import argparse, subprocess

#lib_old = '/usr/local/lib/libliquid.ar'
lib_old = '../liquid-dsp-1.3.2/libliquid.ar'
lib_new = 'libliquid.ar'
opts = '-A'

def nmparse(lib,opts='-A',filt='T'):
    '''run command-line 'nm' program and parse results, filtering out unwanted symbols'''
    out = subprocess.check_output(['nm',opts,lib]).decode('utf-8').split('\n')[:-1]
    out = filter(lambda v: v.split()[-2] in filt, out)
    return set(v.split()[-1] for v in out)

syms_old = nmparse(lib_old)
syms_new = nmparse(lib_new)

# compute removed & added symbols
syms_removed = syms_old.difference(syms_new)
syms_added   = syms_new.difference(syms_old)

# print basic report
print('%32s [old] : %u symbols' % (lib_old,  len(syms_old)))
print('%32s [new] : %u symbols' % (lib_new,  len(syms_new)))
print('      %32s : %u symbols' % ('removed',len(syms_removed)))
print('      %32s : %u symbols' % ('added',  len(syms_added)))

#print('removed:', syms_removed)
#print('added:', syms_added)

print('removed:')
for s in sorted(syms_removed):
    print(' ',s)



#!/usr/bin/python

'''
This script updates the copyright license lines for all files within a
particular directory.

usage:
  $ python scripts/update_copyright.py <dir>
'''

import os
import sys
import re

def usage():
    print __doc__

if len(sys.argv) != 2:
    sys.stderr.write("error: please provide an input directory\n")
    usage()
    sys.exit(-1)
elif sys.argv[1] in ('-h','--help'):
    usage()
    sys.exit(0)

# new copyright
newcopy = '''\
Copyright (c) 2007 - 2015 Joseph Gaeddert

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.'''.splitlines()

# add new line character to end of each line
for i in xrange(len(newcopy)):
    newcopy[i] += '\n'

rootdir = sys.argv[1]

# specific files to ignore
ignore_files = ['update_copyright.py',]

# directories to ignore
ignore_directories = ['.git',]

# only look at these extensions
include_extensions = ['.h', '.c', '.md', '.tex', '.ac', '.in', '.m',]

# print alignment for status
align = 56

#
def update_copyright(filename=""):
    """
    Update copyright on file.
    """
    # try to open file
    contents = []
    with open( filename, 'r' ) as f:
        # read lines from file (removing newline character at end)
        for line in f:
            #contents.append(line.strip('\n\r'))
            contents.append(line)

    # parse contents
    index_start = -1
    index_stop  = -1

    # search for copyright; starts at top of file
    for i in range(min(10, len(contents))):
        if re.search(r'Copyright .* Joseph Gaeddert',contents[i]):
            index_start = i
            break

    if index_start == -1:
        print "  " + filename + ":" + " "*(align-len(filename)) + "no copyright found"
        return

    # look for end of copyright
    #for i in range(index_start+15,index_start+15+min(10, len(contents))):
    i = index_start + 15
    if re.search(r'along with liquid.  If not, see',contents[i]):
        index_stop = i
    elif re.search(r'AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM',contents[i]):
        print "  " + filename + ":" + " "*(align-len(filename)) + "copyright already updated"
        return
    else:
        print "  " + filename + ":" + " "*(align-len(filename)) + "could not find end of copyright"
        return

    # check comment type
    m = re.search(r'^( \*|#+) +Copyright',contents[index_start])
    if m:
        comment = m.group(1)
    else:
        raise Exception('unexpected error')

    # delete items in list
    contents.__delslice__(index_start, index_stop+1)

    # insert new copyright
    for i in range(len(newcopy)):
        # only add space after comment characters if string is not empty
        # (e.g. print ' *' instead of ' * ')
        space = ' ' if len(newcopy[i].strip()) > 0 else ''

        # insert new comment
        contents.insert(index_start+i, comment + space + newcopy[i])

    # open original file for writing
    with open( filename, 'w' ) as f:
        for line in contents:
            f.write(line)
            
    print "  " + filename + ":" + " "*(align-len(filename)) + "updated"

#
for root, subFolders, files in os.walk(rootdir):

    # strip off leading './' if it exists
    if root.startswith('./'):
        root = root[2:]

    # print root directory
    print root
    
    # ignore certain directories
    if root in ignore_directories:
        print "(skipping directory)"
        continue

    # print subfolders
    #for folder in subFolders:
    #    print "%s has subdirectory %s" % (root, folder)

    # parse each filename in directory
    for filename in files:
        filePath = os.path.join(root, filename)

        # check filename
        if filePath in ignore_files:
            print "  " + filePath + ":" + " "*(align-len(filePath)) + "ignoring this specific file"
            continue;

        # check filename extension
        baseName, extension = os.path.splitext(filename)
        if extension not in include_extensions:
            print "  " + filePath + ":" + " "*(align-len(filePath)) + "improper extension; ignoring file"
            continue;

        # continue on with this file
        update_copyright(filePath)





#!/usr/bin/env python3
'''plot frame sync debug files'''
import argparse, os, sys
import numpy as np, matplotlib.pyplot as plt

def main(argv=None):
    p = argparse.ArgumentParser(description=__doc__)
    p.add_argument('sources', nargs='+', help='input files')
    p.add_argument('-export', default=None, action='store_true', help='export figure')
    p.add_argument('-nodisplay', action='store_true', help='disable display')
    args = p.parse_args()

    for fname in args.sources:
        filename = framesync64_plot(fname,args.export)

def framesync64_plot(filename,export=None):
    # open file and read values
    fid         = open(filename,'rb')
    buf         = np.fromfile(fid, count=1440, dtype=np.csingle)
    tau_hat     = np.fromfile(fid, count=   1, dtype=np.single)
    dphi_hat    = np.fromfile(fid, count=   1, dtype=np.single)
    phi_hat     = np.fromfile(fid, count=   1, dtype=np.single)
    gamma_hat   = np.fromfile(fid, count=   1, dtype=np.single)
    evm         = np.fromfile(fid, count=   1, dtype=np.single)
    payload_rx  = np.fromfile(fid, count= 630, dtype=np.csingle)
    payload_sym = np.fromfile(fid, count= 600, dtype=np.csingle)
    payload_dec = np.fromfile(fid, count=  72, dtype=np.int8)

    # compute filter response in dB
    nfft = 2400
    f = np.arange(nfft)/nfft-0.5
    psd = np.abs(np.fft.fftshift(np.fft.fft(buf, nfft)))**2
    m   = int(0.01*nfft)
    w   = np.hamming(2*m+1)
    h   = np.concatenate((w[m:], np.zeros(nfft-2*m-1), w[:m])) / (sum(w) * nfft)
    H   = np.fft.fft(h)
    psd = 10*np.log10( np.real(np.fft.ifft(H * np.fft.fft(psd))) )
    #psd = 20*np.log10(np.abs(np.fft.fftshift(np.fft.fft(buf, nfft))))

    # plot impulse and spectral responses
    fig, _ax = plt.subplots(2,2,figsize=(12,12))
    ax = _ax.flatten()
    t = np.arange(len(buf))
    qpsk = np.exp(0.5j*np.pi*(np.arange(4)+0.5)) # payload constellation
    ax[0].plot(t,np.real(buf), t,np.imag(buf))
    ax[0].set_title('Raw I/Q Samples')
    ax[1].plot(f,psd)
    ax[1].set(xlim=(-0.5,0.5))
    ax[1].set_title('Power Spectral Density')
    ax[2].plot(np.real(payload_rx),  np.imag(payload_rx),  '.')
    ax[2].set_title('RX Payload Syms')
    ax[3].plot(np.real(payload_sym), np.imag(payload_sym), '.')
    ax[3].set_title('Synchronized Payload Syms')
    for _ax in ax[2:]:
        _ax.set(xlim=(-1.3,1.3), ylim=(-1.3,1.3))
        _ax.plot(np.real(qpsk),np.imag(qpsk),'rx')
        _ax.set_xlabel('Real')
        _ax.set_ylabel('Imag')
    for _ax in ax:
        _ax.grid(True)
    fig.suptitle('frame64, tau:%.6f, dphi:%.6f, phi:%.6f, rssi:%.3f dB, evm:%.3f' % \
        (tau_hat, dphi_hat, phi_hat, 20*np.log10(gamma_hat), evm))
    if export==None:
        plt.show()
    else:
        fig.savefig(os.path.splitext(filename)[0]+'.png',bbox_inches='tight')
    plt.close()

if __name__ == '__main__':
    sys.exit(main())




#!/usr/bin/env python3
'''Compare benchmark results'''
import argparse, json, numpy as np

# parse command-line arguments
p = argparse.ArgumentParser(description=__doc__)
p.add_argument('old', help='old benchmark file (.json)')
p.add_argument('new', help='new benchmark file (.json)')
p.add_argument('-thresh', default=1.5, help='threshold for displaying deltas')
args = p.parse_args()

# load json files
v0 = json.load(open(args.old,'r'))
v1 = json.load(open(args.new,'r'))

# convert to dictionary and get set of common names
b0 = {v['name']:v for v in v0['benchmarks']}
b1 = {v['name']:v for v in v1['benchmarks']}
set_common = set(b0.keys()).intersection(set(b1.keys()))

# iterate over common benchmarks and print results
for key in set_common:
    r0,r1 = b0[key], b1[key]
    if 0 in (r0['trials'],r1['trials']):
        continue
    rate = r1['rate'] / r0['rate']
    if np.exp(np.abs(np.log(rate))) < args.thresh:
        continue

    print(' %32s, rate = %12.8f' % (key, rate))

